/*
 * PSP Software Development Kit - http://www.pspdev.org
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPSDK root for details.
 *
 * main.c - Simple elf based network example.
 *
 * Copyright (c) 2005 James F <tyranid@gmail.com>
 * Some small parts (c) 2005 PSPPet
 *
 * $Id: main.c 1661 2006-01-08 13:36:35Z tyranid $
 * $HeadURL: svn://svn.pspdev.org/psp/trunk/pspsdk/src/samples/net/simple/main.c $
 */
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspsdk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pspnet.h>
#include <pspnet_inet.h>
#include <pspnet_apctl.h>
#include <pspnet_resolver.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/select.h>
#include <errno.h>
#include <curl/curl.h>

#define printf pspDebugScreenPrintf

#define MODULE_NAME "NetSample"
#define HELLO_MSG   "Hello there. Type away.\r\n"

PSP_MODULE_INFO(MODULE_NAME, 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common)
{
	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);
	sceKernelSleepThreadCB();

	return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread,
				     0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}


size_t write_data(void *ptr, size_t size, size_t nmemb, void *stream);


/* uses curl to get a page */
void run_curl_test()
{
	CURL *curl;
	CURLcode res;

	pspDebugScreenPrintf("GONNA CURL!\n");

	sceKernelDelayThread(1000*1000*1);

	curl = curl_easy_init();
	if(curl)
	{
		//set url
		curl_easy_setopt(curl, CURLOPT_URL, "http://localhost.geek.nz");

		//no prog meter (i dont think this matters)
		curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
		//shut up completely (i dont think this matters)
		curl_easy_setopt(curl, CURLOPT_MUTE, 1);

		//send data here (this matters)
		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);

		//tell curl to actually get the page etc etc
		res = curl_easy_perform(curl);

		// always cleanup
		curl_easy_cleanup(curl);
	}
	pspDebugScreenPrintf("CURLED! res:%i\n", res);
	sceKernelDelayThread(1000*1000*5);

}


/* callback for curl to return data */
/* we just spam it to the screen, badly coded ;) */
size_t write_data(void *ptr, size_t size, size_t nmemb, void *stream)
{
    int a;
    char* buf = (char*)ptr;

//  pspDebugScreenPrintf("Received %i bytes\n", size*nmemb);
    for (a = 0; a < (size * nmemb); a++)
    {
        pspDebugScreenPrintf("%c", buf[a]);
    }
    return nmemb;
}


/* Connect to an access point */
int connect_to_apctl(int config)
{
	int err;
	int stateLast = -1;

	/* Connect using the first profile */
	err = sceNetApctlConnect(config);
	if (err != 0)
	{
		printf(MODULE_NAME ": sceNetApctlConnect returns %08X\n", err);
		return 0;
	}

	printf(MODULE_NAME ": Connecting...\n");
	while (1)
	{
		int state;
		err = sceNetApctlGetState(&state);
		if (err != 0)
		{
			printf(MODULE_NAME ": sceNetApctlGetState returns $%x\n", err);
			break;
		}
		if (state > stateLast)
		{
			printf("  connection state %d of 4\n", state);
			stateLast = state;
		}
		if (state == 4)
			break;  // connected with static IP

		// wait a little before polling again
		sceKernelDelayThread(50*1000); // 50ms
	}
	printf(MODULE_NAME ": Connected!\n");

	if(err != 0)
	{
		return 0;
	}

	return 1;
}

int net_thread(SceSize args, void *argp)
{
	int err;

	do
	{
		if((err = pspSdkInetInit()))
		{
			printf(MODULE_NAME ": Error, could not initialise the network %08X\n", err);
			break;
		}

		if(connect_to_apctl(1))
		{
			// connected, get my IPADDR and run test
			char szMyIPAddr[32];
			if (sceNetApctlGetInfo(8, szMyIPAddr) != 0)
				strcpy(szMyIPAddr, "unknown IP address");

			run_curl_test();
		}
	}
	while(0);

	return 0;
}

/* Simple thread */
int main(int argc, char **argv)
{
	SceUID thid;

	SetupCallbacks();

	pspDebugScreenInit();

	if(pspSdkLoadInetModules() < 0)
	{
		printf("Error, could not load inet modules\n");
		sceKernelSleepThread();
	}

	/* Create a user thread to do the real work */
	thid = sceKernelCreateThread("net_thread", net_thread, 0x18, 0x10000, PSP_THREAD_ATTR_USER, NULL);
	if(thid < 0)
	{
		printf("Error, could not create thread\n");
		sceKernelSleepThread();
	}

	sceKernelStartThread(thid, 0, NULL);

	sceKernelExitDeleteThread(0);

	return 0;
}
